require 'rubygems'
require 'erb'
require 'zip'
require 'stringio'

class SpreadsheetErb

  def initialize(template_dir:, context:, temp_dir: Dir.mktmpdir)
    @template_dir = template_dir
    @context = context
    @temp_dir = temp_dir
  end
  
  def render
    copy_template()
    xml_paths = find_xml_paths(@temp_dir)
    xml_paths.each do |xml_path|
      result = render_file(xml_path)
      save_file(xml_path, result)
    end
  end

  def write(result_path = Tempfile.new(['result', '.xlsx']).path)
    File.delete(result_path) if File.exist?(result_path)
    zip(@temp_dir, result_path)
    get_file(result_path)
  end


  private

  def get_file(file_path)
    f = File.read(file_path, mode: 'r')
    StringIO.new(f, 'rb')
  end

  def render_file(file_path)
    template = File.read(file_path)
    renderer = ERB.new(template)
    result = renderer.result(@context)
    result
  end

  def save_file(file_path, value)
    File.write(file_path, value)
  end

   def find_xml_paths(template_dir)
    Dir.glob("#{template_dir}/**/*.xml")
  end

  def copy_template
    FileUtils.cp_r("#{@template_dir}/.", @temp_dir)
  end

  def zip(directory_to_zip, result_path)
    zf = ZipFileGenerator.new(directory_to_zip, result_path)
    zf.write()
  end
end

# This is a simple example which uses rubyzip to
# recursively generate a zip file from the contents of
# a specified directory. The directory itself is not
# included in the archive, rather just its contents.
#
# Usage:
#   directory_to_zip = "/tmp/input"
#   output_file = "/tmp/out.zip"
#   zf = ZipFileGenerator.new(directory_to_zip, output_file)
#   zf.write()

class ZipFileGenerator
  # Initialize with the directory to zip and the location of the output archive.
  def initialize(input_dir, output_file)
    @input_dir = input_dir
    @output_file = output_file
  end

  # Zip the input directory.
  def write
    entries = Dir.entries(@input_dir) - %w(. ..)

    ::Zip::File.open(@output_file, ::Zip::File::CREATE) do |zipfile|
      write_entries entries, '', zipfile
    end
  end

  private

  # A helper method to make the recursion work.
  def write_entries(entries, path, zipfile)
    entries.each do |e|
      zipfile_path = path == '' ? e : File.join(path, e)
      disk_file_path = File.join(@input_dir, zipfile_path)
      #puts "Deflating #{disk_file_path}"

      if File.directory? disk_file_path
        recursively_deflate_directory(disk_file_path, zipfile, zipfile_path)
      else
        put_into_archive(disk_file_path, zipfile, zipfile_path)
      end
    end
  end

  def recursively_deflate_directory(disk_file_path, zipfile, zipfile_path)
    zipfile.mkdir zipfile_path
    subdir = Dir.entries(disk_file_path) - %w(. ..)
    write_entries subdir, zipfile_path, zipfile
  end

  def put_into_archive(disk_file_path, zipfile, zipfile_path)
    zipfile.get_output_stream(zipfile_path) do |f|
      f.write(File.open(disk_file_path, 'rb').read)
    end
  end
end